"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Joi = require("joi");
var itErrors = require("./locale/joi_it");
/**
 * SchemaBuilder for the Joi validator.
 * It exposes some facility methods for some simple task,
 * and it also allows a complete personalization of the final result.
 */
var SchemaBuilder = /** @class */ (function () {
    function SchemaBuilder() {
        this.keys = {};
    }
    /**
     * Generate the final Joi Object Schema
     */
    SchemaBuilder.prototype.build = function () {
        return Joi.object().keys(this.keys);
    };
    /**
     * General method, that can be used with the Joi functions
     */
    SchemaBuilder.prototype.general = function (key, spec) {
        this.keys[key] = spec;
        this.lastKey = key;
        return this;
    };
    /**
     * Add an optional string to the Schema
     * @param key the key
     * @param min minimum length of the string
     * @param max maximum length of the string
     * @return the schema builder
     */
    SchemaBuilder.prototype.stringOptional = function (key, min, max) {
        var tmp = Joi.string();
        if (min !== undefined) {
            tmp = tmp.min(min);
        }
        else {
            tmp = tmp.allow("");
        }
        if (max !== undefined) {
            tmp = tmp.max(max);
        }
        this.keys[key] = tmp;
        this.lastKey = key;
        return this;
    };
    /**
     * Add a required string to the Schema
     * @param key the key
     * @param min minimum length of the string
     * @param max maximum length of the string
     * @return the schema builder
     */
    SchemaBuilder.prototype.string = function (key, min, max) {
        this.stringOptional(key, min, max);
        this.keys[key] = this.keys[key].required();
        this.lastKey = key;
        return this;
    };
    /**
     * Add a required email to the Schema
     * @param key the key
     * @return the schema builder
     */
    SchemaBuilder.prototype.email = function (key) {
        this.keys[key] = Joi.string()
            .email()
            .required();
        this.lastKey = key;
        return this;
    };
    /**
     * Add an optional email to the Schema
     * @param key the key
     * @return the schema builder
     */
    SchemaBuilder.prototype.emailOptional = function (key) {
        this.keys[key] = Joi.string().email();
        this.lastKey = key;
        return this;
    };
    /**
     * Add a required password to the Schema.
     * The password will be validated using the following regex:
     * ^[0-9a-zA-Z\|\!\"\£\$\%\&\/\(\)\=\?\^\,\;\.\:\-\_\\\~]{6,}$
     * @param key the key
     * @return the schema builder
     */
    SchemaBuilder.prototype.password = function (key) {
        this.keys[key] = Joi.string()
            .regex(/^[0-9a-zA-Z\|\!\"\£\$\%\&\/\(\)\=\?\^\,\;\.\:\-\_\\\~]{6,}$/)
            .required();
        return this;
    };
    /**
     * Add an optional password to the Schema.
     * The password will be validated using the following regex:
     * ^[0-9a-zA-Z\|\!\"\£\$\%\&\/\(\)\=\?\^\,\;\.\:\-\_\\\~]{6,}$
     * @param key the key
     * @return the schema builder
     */
    SchemaBuilder.prototype.passwordOptional = function (key) {
        this.keys[key] = Joi.string().regex(/^[0-9a-zA-Z\|\!\"\£\$\%\&\/\(\)\=\?\^\,\;\.\:\-\_\\\~]{6,}$/);
        return this;
    };
    /**
     * Add an optional number to the Schema
     * @param key the key
     * @param min minimum value of the number
     * @param max maximum value of the number
     * @return the schema builder
     */
    SchemaBuilder.prototype.numberOptional = function (key, min, max) {
        var tmp = Joi.number();
        if (min !== undefined) {
            tmp = tmp.min(min);
        }
        if (max !== undefined) {
            tmp = tmp.max(max);
        }
        this.keys[key] = tmp;
        this.lastKey = key;
        return this;
    };
    /**
     * Add a required number to the Schema
     * @param key the key
     * @param min minimum value of the number
     * @param max maximum value of the number
     * @return the schema builder
     */
    SchemaBuilder.prototype.number = function (key, min, max) {
        this.numberOptional(key, min, max);
        this.keys[key] = this.keys[key].required();
        this.lastKey = key;
        return this;
    };
    /**
     * Add an optional integer number to the Schema
     * @param key the key
     * @param min minimum value of the number
     * @param max maximum value of the number
     * @return the schema builder
     */
    SchemaBuilder.prototype.integerOptional = function (key, min, max) {
        var tmp = Joi.number().integer();
        if (min !== undefined) {
            tmp = tmp.min(min);
        }
        if (max !== undefined) {
            tmp = tmp.max(max);
        }
        this.keys[key] = tmp;
        this.lastKey = key;
        return this;
    };
    /**
     * Add a required integer number to the Schema
     * @param key the key
     * @param min minimum value of the number
     * @param max maximum value of the number
     * @return the schema builder
     */
    SchemaBuilder.prototype.integer = function (key, min, max) {
        this.integerOptional(key, min, max);
        this.keys[key] = this.keys[key].required();
        this.lastKey = key;
        return this;
    };
    /**
     * Add an optional date to the Schema
     * @param key the key
     * @return the schema builder
     */
    SchemaBuilder.prototype.dateOptional = function (key) {
        this.keys[key] = Joi.date();
        this.lastKey = key;
        return this;
    };
    /**
     * Add a required date to the Schema
     * @param key the key
     * @return the schema builder
     */
    SchemaBuilder.prototype.date = function (key) {
        this.keys[key] = Joi.date().required();
        this.lastKey = key;
        return this;
    };
    SchemaBuilder.prototype.arrayOfStrings = function (key) {
        this.keys[key] = Joi.array().items(Joi.string());
        this.lastKey = key;
        return this;
    };
    SchemaBuilder.prototype.arrayOfStringsOptional = function (key) {
        this.keys[key] = Joi.array().items(Joi.string());
        this.lastKey = key;
        return this;
    };
    /**
     * Add the label to the last added key
     * @param label the label to use in case of error
     * @return the schema builder
     */
    SchemaBuilder.prototype.withLabel = function (label) {
        if (this.lastKey) {
            this.keys[this.lastKey] = this.keys[this.lastKey].label(label);
        }
        return this;
    };
    return SchemaBuilder;
}());
exports.SchemaBuilder = SchemaBuilder;
/**
 * This class is used to validate an object using a given schema.
 * It is used by Lynx to automatically validate the body of any requests, using
 * the Body decodator.
 */
var ValidateObject = /** @class */ (function () {
    /**
     * @param obj the object to validate
     * @param schema the schema
     * @param locales an array of available language. You can use the `req.acceptsLanguages()`
     */
    function ValidateObject(obj, schema, locales) {
        this._obj = obj;
        this.schema = schema;
        var options = null;
        for (var _i = 0, locales_1 = locales; _i < locales_1.length; _i++) {
            var locale = locales_1[_i];
            if (locale.indexOf("en") != -1) {
                break;
            }
            if (locale == "it") {
                options = {
                    language: itErrors.errors
                };
                break;
            }
        }
        this.validate(options);
    }
    ValidateObject.prototype.validate = function (options) {
        this.valid = Joi.validate(this._obj, this.schema, options);
    };
    Object.defineProperty(ValidateObject.prototype, "isValid", {
        /**
         * Verify that the object rescpect the schema.
         * @return true if the object is valid, false otherwise.
         */
        get: function () {
            return this.valid.error === null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValidateObject.prototype, "obj", {
        /**
         * Unwrap the object (can be valid or not!)
         * @return the unwrapped object
         */
        get: function () {
            return this._obj;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValidateObject.prototype, "errors", {
        /**
         * Getter that returns an array of validation errors.
         * @return an array of validation errors. It can not be null.
         */
        get: function () {
            var errors = [];
            if (this.isValid) {
                return errors;
            }
            for (var _i = 0, _a = this.valid.error.details; _i < _a.length; _i++) {
                var err = _a[_i];
                errors.push({
                    name: err.context && err.context.key ? err.context.key : "",
                    message: err.message
                });
            }
            return errors;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValidateObject.prototype, "errorsMap", {
        /**
         * Getter that returns a map of errors. This prop contains the save information
         * as the `errors` prop, but with a different format.
         * @return a map or loclaized errors.
         */
        get: function () {
            var map = {};
            for (var _i = 0, _a = this.errors; _i < _a.length; _i++) {
                var err = _a[_i];
                map[err.name] = err.message;
            }
            return map;
        },
        enumerable: true,
        configurable: true
    });
    return ValidateObject;
}());
exports.ValidateObject = ValidateObject;
